import logging
from django.contrib.auth.signals import user_logged_in
from django.dispatch import receiver
from rest_framework_jwt.authentication import JSONWebTokenAuthentication


logger = logging.getLogger(__name__)


@receiver(user_logged_in)
def login_handler(sender, user, request, **kwargs):
    logger.debug("%r logged in." % user)
    request.db.login(user)


class JWTAuthentication(JSONWebTokenAuthentication):
    def authenticate(self, request):
        result = super().authenticate(request)

        if result is None:
            return

        user, token = result
        user_logged_in.send(sender=user.__class__, request=request, user=user)

        return (user, token)
