import logging
from sqlalchemy.exc import DBAPIError as DatabaseError
from sqlalchemy.orm.attributes import ScalarObjectAttributeImpl
from rest_framework.decorators import api_view, renderer_classes
from rest_framework.response import Response
from rest_framework.renderers import JSONRenderer
from gargantext.models import ProjectNode


logger = logging.getLogger(__name__)


# https://bitbucket.org/zzzeek/sqlalchemy/issues/3976/built-in-way-to-convert-an-orm-object-ie
from sqlalchemy import inspect
def orm_is_value(a):
    return hasattr(a, 'key') and \
        not isinstance(getattr(a, 'impl'), ScalarObjectAttributeImpl)
def orm_to_dict(obj):
    attrs = inspect(type(obj)).all_orm_descriptors
    return {a.key: getattr(obj, a.key) for a in attrs if orm_is_value(a)}


@api_view(['GET'])
@renderer_classes((JSONRenderer,))
def projects_view(request):
    try:
        r = list(map(orm_to_dict, request.db.query(ProjectNode).all()))
    except DatabaseError as e:
        logger.debug("Error: %s" % e)
        r = []

    return Response({'results': r})
